"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const api_1 = __importDefault(require("@mesg/api"));
const orchestrator_1 = __importDefault(require("@mesg/orchestrator"));
const sort_object_1 = __importDefault(require("@mesg/api/lib/util/sort-object"));
const transaction_1 = __importDefault(require("@mesg/api/lib/transaction"));
const account_1 = __importDefault(require("@mesg/api/lib/account"));
class Runner {
    constructor(provider, lcdEndpoint, orchestratorEndpoint, mnemonic, engineAddress) {
        this._api = new api_1.default(lcdEndpoint);
        this._orchestrator = new orchestrator_1.default(orchestratorEndpoint);
        this._provider = provider;
        this._mnemonic = mnemonic;
        this._engineAddress = engineAddress;
    }
    start(serviceHash, env) {
        return __awaiter(this, void 0, void 0, function* () {
            const service = yield this._api.service.get(serviceHash);
            const { runnerHash, instanceHash, envHash } = yield this._api.runner.hash(this._engineAddress, serviceHash, env);
            const envObj = (env || []).reduce((prev, x) => (Object.assign(Object.assign({}, prev), { [x.split('=')[0]]: x.split('=')[1] })), {});
            yield this._provider.start(service, Object.assign(Object.assign({}, envObj), { MESG_ENDPOINT: `engine:50052`, MESG_SERVICE_HASH: service.hash, MESG_ENV_HASH: envHash, MESG_REGISTER_SIGNATURE: this.sign({
                    serviceHash: serviceHash,
                    envHash: envHash
                }) }), runnerHash);
            return {
                instanceHash,
                hash: runnerHash
            };
        });
    }
    stop(runnerHash) {
        return __awaiter(this, void 0, void 0, function* () {
            const runner = yield this._api.runner.get(runnerHash);
            const instance = yield this._api.instance.get(runner.instanceHash);
            const payload = { runnerHash };
            yield this._orchestrator.runner.delete(payload, this.sign(payload));
            yield this._provider.stop(runnerHash, instance.serviceHash);
        });
    }
    sign(data) {
        const ecdsa = transaction_1.default.sign(JSON.stringify(sort_object_1.default(data)), account_1.default.getPrivateKey(this._mnemonic));
        return Buffer.from(ecdsa.signature).toString('base64');
    }
}
exports.default = Runner;
