import { Docker } from "node-docker-api";
import { Network } from "node-docker-api/lib/network";
import { Container as DockerContainer } from "node-docker-api/lib/container";
export declare type IContainer = {
    Image?: string;
    ExposedPorts?: {
        [key: string]: Object;
    };
    Labels?: {
        [key: string]: string;
    };
    Env?: string[];
    Command?: string;
    Args?: string[];
    HostConfig?: {
        PortBindings?: {
            [key: string]: {
                HostPort: string;
            }[];
        };
        RestartPolicy?: {
            Name: 'on-failure' | 'always' | 'unless-stopped';
            MaximumRetryCount?: number;
        };
        Mounts?: {
            Source: string;
            Target: string;
            Type: 'bind' | 'volume';
        }[];
    };
    NetworkingConfig?: {
        EndpointsConfig?: {
            [key: string]: {
                Aliases: string[];
            };
        };
    };
};
export default class Container {
    private client;
    private _containerInfo;
    private _networks;
    private _name;
    static findAll(client: any, labels: {
        [key: string]: string;
    }): Promise<DockerContainer[]>;
    static hasImage(client: any, image: string): Promise<boolean>;
    static fetchImage(client: any, image: string): Promise<void>;
    constructor(options: IContainer, name: string, client: Docker);
    addPorts(ports: string[]): Container;
    connectTo(network: Network, aliases: string[]): Container;
    start(): Promise<DockerContainer>;
}
