"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class Container {
    constructor(options, name, client) {
        this._containerInfo = {};
        this._networks = [];
        this._containerInfo = options;
        this._name = name;
        this.client = client;
    }
    static findAll(client, labels) {
        return __awaiter(this, void 0, void 0, function* () {
            const allLabels = (data) => Object.keys(labels)
                .reduce((prev, x) => prev && data[x] === labels[x], true);
            const list = yield client.container.list({ all: true });
            return list.filter(x => allLabels(x.data['Labels'] || {}));
        });
    }
    static hasImage(client, image) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield client.image.get(image).status();
                return true;
            }
            catch (e) {
                if (e.statusCode !== 404)
                    throw e;
                return false;
            }
        });
    }
    static fetchImage(client, image) {
        return __awaiter(this, void 0, void 0, function* () {
            const streamToPromise = (stream) => new Promise((resolve, reject) => stream
                .on('data', (x) => x.toString()) // For some reason we need to listen to the data to have the end event
                .on('error', reject)
                .on('end', resolve));
            const [fromImage, tag] = image.split(':');
            const stream = yield client.image.create({}, {
                fromImage,
                tag: tag || 'latest'
            });
            yield streamToPromise(stream);
        });
    }
    // https://stackoverflow.com/questions/20428302/binding-a-port-to-a-host-interface-using-the-rest-api/20429133
    addPorts(ports) {
        if (!this._containerInfo.ExposedPorts)
            this._containerInfo.ExposedPorts = {};
        if (!this._containerInfo.HostConfig)
            this._containerInfo.HostConfig = {};
        if (!this._containerInfo.HostConfig.PortBindings)
            this._containerInfo.HostConfig.PortBindings = {};
        for (const port of ports || []) {
            this._containerInfo.ExposedPorts[`${port}/tcp`] = {};
            this._containerInfo.HostConfig.PortBindings[`${port}/tcp`] = [{ HostPort: port }];
        }
        return this;
    }
    // https://stackoverflow.com/questions/60301221/docker-container-cannot-be-connected-to-network-endpoints
    connectTo(network, aliases) {
        if (!this._containerInfo.NetworkingConfig)
            this._containerInfo.NetworkingConfig = {};
        if (!this._containerInfo.NetworkingConfig.EndpointsConfig)
            this._containerInfo.NetworkingConfig.EndpointsConfig = {};
        if (Object.keys(this._containerInfo.NetworkingConfig.EndpointsConfig).length === 0) {
            this._containerInfo.NetworkingConfig.EndpointsConfig[network.id] = {
                Aliases: aliases
            };
        }
        else {
            this._networks.push(network);
        }
        return this;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(yield Container.hasImage(this.client, this._containerInfo.Image))) {
                yield Container.fetchImage(this.client, this._containerInfo.Image);
            }
            const containers = yield Container.findAll(this.client, this._containerInfo.Labels);
            const container = containers.length === 1
                ? containers[0]
                : yield this.client.container.create(Object.assign(Object.assign({}, this._containerInfo), { name: this._name }));
            for (const network of this._networks) {
                if ((yield network.status()).data.Containers[container.id])
                    continue;
                yield network.connect({ container: container.id });
            }
            return container.start();
        });
    }
}
exports.default = Container;
